//! Hash and salt functions

use argon2;
use rand::distributions::{Distribution, Uniform};

use crate::result::RudeResult;
use crate::try_log;

/// Compute an argon2 hash
///
/// # Arguments
///
/// * data: string to hash
/// * salt: string to salt the hash with
///
/// # Example
///
/// ```
/// use rude::hash::*;
/// let data = "butterflies";
/// let salt = salt();
/// println!("{}", hash(data, salt));
/// ```
pub fn hash<D: Into<String>, S: Into<String>>(data: D, salt: S) -> RudeResult<String> {
	Ok(try_log!(
		argon2::hash_encoded(
			data.into().as_bytes(),
			salt.into().as_bytes(),
			&argon2::Config::default(),
		),
		"Unable to create hash",
	))
}

/// Generate a random salt
///
/// # Example
///
/// ```
/// use rude::hash::salt;
/// let salt = salt();
/// println!("{}", salt);
/// ```
pub fn salt() -> String {
	let between = Uniform::from(32..127);
	let mut rng = rand::thread_rng();
	let mut salt = String::new();

	for _ in 0..64 {
		salt.push(char::from(between.sample(&mut rng)));
	}

	salt
}
